// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  TU Dresden                                                            ==
// ==                                                                        ==
// ==  Institut fr Wissenschaftliches Rechnen                               ==
// ==  Zellescher Weg 12-14                                                  ==
// ==  01069 Dresden                                                         ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  https://gforge.zih.tu-dresden.de/projects/amdis/                      ==
// ==                                                                        ==
// ============================================================================

/** \file CoarseningManager2d.h */

#ifndef AMDIS_COARSENINGMANAGER_2D_H
#define AMDIS_COARSENINGMANAGER_2D_H

#include "CoarseningManager.h"

namespace AMDiS {

  /** \ingroup Adaption  
   * \brief
   * Implements a CoarseningManager for 2-dimensional meshes.
   */
  class CoarseningManager2d : public CoarseningManager
  {
  public:

    /// Calls base class constructor and checks dimension of mesh. 
    CoarseningManager2d() 
      : CoarseningManager() 
    {}

    /// destructor
    virtual ~CoarseningManager2d() {}

  protected:
    /// Implements CoarseningManager::coarsenFunction
    virtual int coarsenFunction(ElInfo *el_info);

    /** \brief
     * Coarsens a single Triangle of the coarsening patch. DOFs
     * in the interior of the element are removed; DOFs for higher order
     * at the boundary or the coarsening patch still belong to 
     * the parent. Do not remove them form the mesh!!!                  
     */
    void coarsenTriangle(Triangle *el);

    /** \brief
     * First rebuild the DOFs on the parents then do restriction
     * of data (if possible) and finally coarsen the patch elements
     */
    void coarsenPatch(RCNeighbourList *coarsenList, int n_neigh, int bound);
  };


}

#endif // AMDIS_COARSENINGMANAGER_2D_H
